/*
 * Copyright 2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @author Andres Almiray
 */

function assertTrue( expr ){
	currentException = null;
	if( isUndefined(expr) ){
		currentException = new JsAssertException("undefined: "+expr);
		throw currentException;
	}
	var bool = eval(expr);
	if( !isBoolean(bool) ){
		currentException = new JsAssertException("not a boolean expression: "+expr);
		throw currentException;
	}
	return bool;
}

function assertNull( expr ){
	currentException = null;
	return isNull(expr);
}

function assertNotNull( expr ){
	currentException = null;
	if( isUndefined(expr) ){
		currentException = new JsAssertException("undefined: "+expr);
		throw currentException;
	}
	if( !isObject(expr) ){
		currentException = new JsAssertException("not an object: "+expr);
		throw currentException;
	}
	return !isNull(expr);
}

function assertEquals( expr1, expr2 ){
	curentException = null;
	if( isUndefined(expr1) ){
		currentException = new JsAssertException("undefined: "+expr1);
		throw currentException;
	}
	if( isUndefined(expr2) ){
		currentException = new JsAssertException("undefined: "+expr2);
		throw currentException;
	}
	if( typeof(expr1) != typeof(expr2) ){
		currentException = new JsAssertException("expression types are different: expr1.type="+typeof(expr1)+" expr2.type="+typeof(expr2));
		throw currentException;	
	}
	if( isArray(expr1) ){
		return assertArrayEquals(expr1, expr2);
	}
	if( isObject(expr1) ){
		return assertObjectEquals(expr1, expr2);
	}
	return expr1 == expr2;
}

function assertObjectEquals( expr1, expr2 ){
	currentException = null;
	if( isUndefined(expr1) ){
		currentException = new JsAssertException("undefined: "+expr1);
		throw currentException;
	}
	if( isUndefined(expr2) ){
		currentException = new JsAssertException("undefined: "+expr2);
		throw currentException;
	}
	if( !isObject(expr1) ){
		currentException = new JsAssertException("not an object: "+expr1);
		throw currentException;
	}
	if( !isObject(expr2) ){
		currentException = new JsAssertException("not an object: "+expr2);
		throw currentException;
	}
	for( prop in expr1 ){
		var aprop = eval("expr1."+prop);
		var bprop = eval("expr2."+prop);
		if( isArray(aprop) && isArray(bprop) ){
			if( !assertArrayEquals(aprop,bprop) ){
				return false;
			}
		}else if( isObject(aprop) && isObject(bprop) ){
			if( !assertObjectEquals(aprop,bprop) ){
				return false;
			}
		}else if( aprop != bprop ){
			return false;
		}
	}
	return true;
}

function assertArrayEquals( expr1, expr2 ){
	currentException = null;
	if( isUndefined(expr1) ){
		currentException = new JsAssertException("undefined: "+expr1);
		throw currentException;
	}
	if( isUndefined(expr2) ){
		currentException = new JsAssertException("undefined: "+expr2);
		throw currentException;
	}
	if( !isArray(expr1) ){
		currentException = new JsAssertException("not an array: "+expr1);
		throw currentException;
	}
	if( !isArray(expr2) ){
		currentException = new JsAssertException("not an array: "+expr2);
		throw currentException;
	}
	if( expr1.length != expr2.length ){
		currentException = new JsAssertException("expr1.length != expr.length: "+expr1.length+" != "+expr2.length);
		throw currentException;
	}
	for( var i = 0; i < expr1.length; i++ ){
		var a = expr1[i];
		var b = expr2[i];
		if( isArray(a) && isArray(b) ){
			if( !assertArrayEquals(a,b) ){
				return false;
			}
		}else if( isObject(a) && isObject(b) ){
			if( !assertObjectEquals(a,b) ){
				return false;
			}
		}
		else if( a != b ){
			return false;
		}
	}
	return true;
}

function JsAssertException( message ){
	this.message = message;
}

var currentException = null;