/*
 * Taken from http://www.crockford.com/javascript/remedial.html
 * @author Douglas Crockford
 */

/**
 * Internet Explorer holds references to objects which are not JavaScript objects, 
 * and which produce errors if they are treated as JavaScript objects. 
 * This is a problem because typeof identifies them as JavaScript objects. 
 * The isAlien() function will return true if a is one of those alien objects.
 */
function isAlien(a) {
   return isObject(a) && typeof a.constructor != 'function';
}

/**
 * Returns true if a is an array, meaning that it was produced by the 
 * Array constructor or by using the [ ] array literal notation.
 */
function isArray(a) {
    return isObject(a) && a.constructor == Array;
}

/**
 * Returns true if a is one of the boolean values, true or false.
 */
function isBoolean(a) {
    return typeof a == 'boolean';
}

/**
 * Returns true if a is an object or array or function containing no enumerable members.
 */
function isEmpty(o) {
    var i, v;
    if (isObject(o)) {
        for (i in o) {
            v = o[i];
            if (isUndefined(v) && isFunction(v)) {
                return false;
            }
        }
    }
    return true;
}

/**
 * Returns true if a is a function. Beware that some native functions in IE were made to look 
 * like objects instead of functions. This function does not detect that.
 */
function isFunction(a) {
    return typeof a == 'function';
}

/**
 * Returns true if a is the null value.
 */
function isNull(a) {
    return typeof a == 'object' && !a;
}

/**
 * Returns true if a is a finite number. It returns false if a is NaN or Infinite. 
 * It also returns false if a is a string that could be converted to a number.
 */
function isNumber(a) {
    return typeof a == 'number' && isFinite(a);
}

/**
 * Returns true if a is an object, and array, or a function. 
 * It returns false if a is a string, a number, a boolean, or null, or undefined.
 */
function isObject(a) {
    return (a && typeof a == 'object') || isFunction(a);
}

/**
 * Returns true if a is a string.
 */
function isString(a) {
    return typeof a == 'string';
}

/**
 * Returns true if a is the undefined value. You can get the undefined value 
 * from an uninitialized variable or from a missing member of an object.
 */
function isUndefined(a) {
    return typeof a == 'undefined';
} 